from typing import Any


class Config:
    _NAME_DOJINESS_THRESHOLD = "DOJINESS_THRESHOLD"

    def __init__(self) -> None:
        self._settings: dict[str, int | float | str | None] = {}
        self.reset()

    def reset(self) -> None:
        self._settings[self._NAME_DOJINESS_THRESHOLD] = 0.95

    @property
    def DOJINESS_THRESHOLD(self) -> float:
        return self._settings.get(Config._NAME_DOJINESS_THRESHOLD)  # type: ignore  # noqa: E501

    @DOJINESS_THRESHOLD.setter
    def DOJINESS_THRESHOLD(self, value: float) -> None:
        self._settings[config._NAME_DOJINESS_THRESHOLD] = value

    def update(self, **kwargs: Any) -> None:
        self._settings.update(kwargs)

    def get(self, key: str) -> int | float | str | None:
        return self._settings.get(key)


# Create a singleton instance of the Config class
config = Config()


def get_config() -> Config:
    global config
    return config


def reset_config() -> None:
    global config
    config.reset()



import numpy as np

from xarizmi.candlestick import Candlestick


def generate_random_candlestick() -> Candlestick:
    nums = np.random.random(4)
    nums.sort()
    nums = nums.tolist()
    return Candlestick(
        close=nums[2],
        open=nums[1],
        low=nums[0],
        high=nums[3],
        volume=np.random.randint(low=100, high=200),
        amount=np.random.randint(low=100, high=200),
    )



"""A client to download Yahoo Finance data
"""

import pandas as pd
import yfinance as yf

from xarizmi.candlestick import CandlestickChart


class YahooFinanceDailyDataClient:

    def __init__(
        self,
        symbol: str,
        start_date: str = "1900-01-01",
        end_date: str | None = None,
    ) -> None:
        self.symbol = symbol
        self.start_date = start_date
        self.end_date = end_date

    def extract(self) -> list[dict[str, str | float | pd.Timestamp]] | None:
        stock_data: pd.DataFrame = yf.download(
            self.symbol, start=self.start_date, end=self.end_date
        )

        if stock_data.empty:
            print("Error fetching data for symbol:", self.symbol)
            return None

        # Reset the index to get 'Date' as a column
        stock_data.reset_index(inplace=True)

        return stock_data.to_dict(orient="records")  # type: ignore

    def transform(
        self, data_list: list[dict[str, str | float | pd.Timestamp]]
    ) -> CandlestickChart:
        candles_data = []
        for single_candle_data in data_list:
            temp = {}
            temp["open"] = single_candle_data["Open"]
            temp["high"] = single_candle_data["High"]
            temp["low"] = single_candle_data["Low"]
            temp["close"] = single_candle_data["Close"]
            temp["volume"] = single_candle_data["Volume"]
            temp["datetime"] = single_candle_data["Date"].to_pydatetime()  # type: ignore  # noqa:E501
            temp["interval_type"] = "1day"
            temp["symbol"] = {
                "base_currency": {"name": self.symbol},
                "quote_currency": {"name": "CAD"},
                "fee_currency": {"name": "CAD"},
            }  # type: ignore
            candles_data.append(temp)
        return CandlestickChart.model_validate({"candles": candles_data})

    def save_file(
        self,
        candlestick_chart: CandlestickChart,
        filepath: str,
        indent: int = 4,
    ) -> None:
        with open(filepath, "w") as f:
            f.write(candlestick_chart.model_dump_json(indent=indent))

    def etl(self, filepath: str) -> CandlestickChart:
        data_list = self.extract()
        if data_list:
            candlestick_chart = self.transform(data_list=data_list)
            self.save_file(
                candlestick_chart=candlestick_chart, filepath=filepath
            )
            return candlestick_chart
        else:
            return CandlestickChart(candles=[])

    @staticmethod
    def download_full_data(symbol: str, filepath: str) -> CandlestickChart:
        client = YahooFinanceDailyDataClient(symbol=symbol)
        candlestick_chart = client.etl(filepath=filepath)
        return candlestick_chart



from datetime import datetime as dt

from pydantic import BaseModel
from pydantic import NonNegativeFloat

from xarizmi.config import config
from xarizmi.enums import IntervalTypeEnum
from xarizmi.models.symbol import Symbol
from xarizmi.utils.extremums import find_local_minima_values
from xarizmi.utils.numbers import round_to_significant_digit


class Candlestick(BaseModel):
    """This class contains basic properties of a candlestick.

    :param close: the closing price for the corresponding time period
    :type close: float
    :param open: the opening price for the corresponding time period
    :type open: float
    :param low: the lowest traded price for the corresponding time period
    :type low: float
    :param high: the highest traded price for the corresponding time period
    :type high: float
    :param volume: the amount of trading volume (width of the candlestick)
    :type volume: float
    :param amount: [amount description?]
    :type amount: float
    :param interval_type: the time period for each candlestick
    :type interval_type: str
    :param symbol: the type of the trade, e.g., "CAD-USD" or "BTC-USDT"
    :type symbol: str
    """

    close: NonNegativeFloat
    open: NonNegativeFloat
    low: NonNegativeFloat
    high: NonNegativeFloat
    volume: NonNegativeFloat
    amount: NonNegativeFloat | None = None
    interval_type: IntervalTypeEnum | None = None
    interval: int | None = None  # interval in seconds
    symbol: Symbol | None = None
    datetime: dt | None = None

    @property
    def is_bullish(self) -> bool:
        if self.close >= self.open:
            return True
        else:
            return False

    @property
    def is_bearish(self) -> bool:
        if self.open >= self.close:
            return True
        else:
            return False

    @property
    def range(self) -> float:
        """Range = H - L"""
        return self.high - self.low

    @property
    def intrinsic_range(self) -> float:
        """
        IR = R / L
        """
        if (self.low) == 0:
            return 0
        else:
            return self.range / (self.low)

    @property
    def body(self) -> float:
        """B = O - C"""
        return abs(self.open - self.close)

    @property
    def intrinsic_body(self) -> float:
        """B / R"""
        if self.range == 0:
            return 0
        return self.body / self.range

    @property
    def upper_shadow(self) -> float:
        """US = H - MAX(C, O)"""
        return self.high - max(self.close, self.open)

    @property
    def intrinsic_upper_shadow(self) -> float:
        """US / R"""
        if self.range == 0:
            return 0
        return self.upper_shadow / self.range

    @property
    def lower_shadow(self) -> float:
        """LS = min(C, O) - L"""
        return min(self.close, self.open) - self.low

    @property
    def intrinsic_lower_shadow(self) -> float:
        """LS / R"""
        if self.range == 0:
            return 0
        return self.lower_shadow / self.range

    @property
    def doginess(self) -> float:
        if self.range == 0:
            return 0
        return 1 - self.intrinsic_body

    @property
    def is_doji(self) -> bool:
        if self.range == 0:
            return False
        return self.doginess >= config.DOJINESS_THRESHOLD


class CandlestickChart(BaseModel):
    candles: list[Candlestick]

    def filter_keep_last_n(
        self, keep_last_n: int | None = None, inplace: bool = False
    ) -> "CandlestickChart":
        if keep_last_n:
            candles = self.candles[-keep_last_n:]
            if inplace is True:
                self.candles = candles
                return self
            else:
                return CandlestickChart(candles=candles)
        return self

    def get_local_minimas(
        self, price_type: str = "low", only_significant_digit: bool = False
    ) -> list[int | float]:
        if price_type not in ["low", "high", "close", "open"]:
            raise ValueError(
                "The given value for price_type ="
                f" '{price_type}' is not valid!"
            )
        values = find_local_minima_values(
            [getattr(candle, price_type) for candle in self.candles]
        )
        if only_significant_digit is True:
            values = [round_to_significant_digit(item) for item in values]
        return values



# Import required functions
from pathlib import Path

from setuptools import find_packages, setup

import xarizmi

# read the contents of your README file
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()


# Call setup function
setup(
    author="Javad Ebadi",
    author_email="javad@javadebadi.com",
    description=(
        "Xarizmi (read Khwarizmi) project is an educational project that"
        "contains tools for technical analysis in Python."
    ),
    name="xarizmi",
    packages=find_packages(include=["xarizmi", "xarizmi.*"]),
    version=xarizmi.__version__,
    install_requires=[],
    python_requires=">=3.11",
    license="Apache 2.0",
    url="https://github.com/javadebadi/xarizmi",
    long_description=long_description,
    long_description_content_type="text/markdown",
)
